from solana.publickey import PublicKey

SERUM_PROGRAM_ID = PublicKey('srmqPvymJeFKQ4zGQed1GFppgkRHL9kaELCbyksJtPX')

RUST_U64_MAX = 18446744073709551615

RUST_I64_MAX = 9223372036854775807

RUST_I64_MIN = -9223372036854775807

QUOTE_DECIMALS = 6
